'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _eos = require('./eos');

var eos = _interopRequireWildcard(_eos);

var _Identity = require('../models/Identity');

var _Identity2 = _interopRequireDefault(_Identity);

var _ClaimReservation = require('../models/ClaimReservation');

var _ClaimReservation2 = _interopRequireDefault(_ClaimReservation);

var _murmurhash = require('murmurhash');

var _murmurhash2 = _interopRequireDefault(_murmurhash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const reservations = 'reservations';
const identities = 'identities';

const getIdentity = (() => {
    var _ref = _asyncToGenerator(function* (table, name) {
        return yield eos.read({
            table: table,
            index: _murmurhash2.default.v2(name.toLowerCase()),
            limit: 1,
            model: _Identity2.default,
            firstOnly: true
        });
    });

    return function getIdentity(_x, _x2) {
        return _ref.apply(this, arguments);
    };
})();

const possibleSuffixes = ['Rabbit', 'Rhino', 'Raccoon', 'Ragdoll', 'Rat', 'Robin', 'Royalty'];

const randomSuffix = () => {
    const suffix = possibleSuffixes[Math.floor(Math.random() * possibleSuffixes.length)];
    const randomNumber = Math.floor(Math.random() * 10000000 + 1);
    return `${suffix}${randomNumber}`;
};

const generateRandomName = () => {
    const name = `Random${randomSuffix()}`;
    if (name.length > 20) return generateRandomName();
    return name;
};

class IdentityService {

    constructor() {}

    validName(name) {
        return (/^[a-zA-Z0-9_-]{3,20}$/.test(name)
        );
    }

    randomName() {
        return generateRandomName();
    }

    randomUniqueName() {
        var _this = this;

        return _asyncToGenerator(function* () {
            let name = generateRandomName();
            while (yield _this.exists(name)) name = generateRandomName();
            return name;
        })();
    }

    get(name) {
        return _asyncToGenerator(function* () {
            return getIdentity(identities, name);
        })();
    }

    reservation(name) {
        return _asyncToGenerator(function* () {
            return getIdentity(reservations, name);
        })();
    }

    identify(name, publicKey) {
        var _this2 = this;

        return _asyncToGenerator(function* () {
            return (yield eos.write()).identify(name, publicKey, eos.authorization).then(function () {
                return _this2.get(name);
            }).catch(function (err) {
                return null;
            });
        })();
    }

    claim(name, signedHash, publicKey) {
        var _this3 = this;

        return _asyncToGenerator(function* () {
            const claim = new _ClaimReservation2.default(name, signedHash, publicKey);
            return (yield eos.write()).claim({ claim, wtv: '' }, eos.authorization).then(function () {
                return _this3.get(name);
            }).catch(function (err) {
                return null;
            });
        })();
    }

    release(name, signedHash) {
        return _asyncToGenerator(function* () {
            return (yield eos.write()).release(name, signedHash, eos.authorization).then(function () {
                return true;
            }).catch(function (err) {
                return false;
            });
        })();
    }

    getHash(name) {
        var _this4 = this;

        return _asyncToGenerator(function* () {
            return (yield _this4.get(name).then(function (x) {
                return x ? x.hash : null;
            })) || (yield _this4.reservation(name).then(function (x) {
                return x ? x.hash : null;
            }));
        })();
    }

    exists(name) {
        var _this5 = this;

        return _asyncToGenerator(function* () {
            return (yield _this5.get(name).then(function (x) {
                return !!x;
            })) || (yield _this5.reservation(name).then(function (x) {
                return !!x;
            }));
        })();
    }

    registered(name) {
        var _this6 = this;

        return _asyncToGenerator(function* () {
            return yield _this6.get(name).then(function (x) {
                return x ? parseInt(x.registered) > 0 : false;
            });
        })();
    }
}
exports.default = IdentityService;